//
//  ActivityRecognitionManager.swift
//  activity_rec_flutter
//
//  Created by WOO JIN HWANG on 2021/04/23.
//

import CoreMotion
import Foundation
import SQLite3

typealias ActivityDataHandler = (ActivityData) -> Void

class ActivityRecognitionManager {
  private let motionActivityManager = CMMotionActivityManager()
  private var isRunningService = false

  public func startService(handler: @escaping ActivityDataHandler) {
    if (isRunningService) {
      NSLog("The activity recognition service has already started.")
      stopService()
    }

    motionActivityManager.startActivityUpdates(to: .main) { (activity) in
      guard let activity = activity else { return }

      let actData = ActivityData(from: activity)


      var db: OpaquePointer? = nil
      let dbPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first! + "/myDatabase.db"

      if sqlite3_open(dbPath, &db) == SQLITE_OK {
          print("sqlite3_open Successfully opened connection to database.")
          let insertStatementString = "INSERT INTO activity (id, name, confidence, lastUpdatedTime) VALUES (?, ?, ?, ?);"
          var insertStatement: OpaquePointer? = nil

          if sqlite3_prepare_v2(db, insertStatementString, -1, &insertStatement, nil) == SQLITE_OK {
              let columnID = String(Date().timeIntervalSince1970 * 1000)
              let columnNAME = actData.type
              let columnConfidence = actData.confidence
              let columnLastUpTime = String(Date().timeIntervalSince1970 * 1000)

              let SQLITE_TRANSIENT = unsafeBitCast(-1, to: sqlite3_destructor_type.self)



              sqlite3_bind_text(insertStatement, 2, columnNAME.cString(using: .utf8), -1, SQLITE_TRANSIENT)
              sqlite3_bind_text(insertStatement, 3, columnConfidence.cString(using: .utf8), -1, SQLITE_TRANSIENT)
              sqlite3_bind_text(insertStatement, 4, columnLastUpTime.cString(using: .utf8), -1, SQLITE_TRANSIENT)

              if sqlite3_step(insertStatement) == SQLITE_DONE {
                  print("Successfully inserted row.")
              } else {
                  let errorMessage = String(cString: sqlite3_errmsg(db)!)
                  print("Failed to insert row. Error: \(errorMessage)")
              }
          } else {
              let errorMessage = String(cString: sqlite3_errmsg(db)!)
              print("Failed to prepare statement. Error: \(errorMessage)")
          }

          sqlite3_finalize(insertStatement)

      } else {
          let errorMessage = String(cString: sqlite3_errmsg(db)!)
          print("sqlite3_open Unable to open database. Error: \(errorMessage)")
      }

      sqlite3_close(db)
      handler(ActivityData(from: activity))
    }
    isRunningService = true
  }
  
  public func stopService() {
    motionActivityManager.stopActivityUpdates()
    isRunningService = false
  }
}
